@@ -174,7 +174,7 @@ class Dataset2D(Dataset):
         
 
 class CustomDataset(Dataset):
-    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = True, resample = None):
+    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = True, resample = None, save_file_list = True):
         self.root_dir = root_dir
         self.image_name = []
         self.info_name = []
@@ -184,6 +184,7 @@ class CustomDataset(Dataset):
         self.dim = dim
         self.resample = resample
         self.correct_direction = correct_direction
+        self.save_file_list = save_file_list
         if resample != None :
             self.resample.device = device
         if normalizer != None:
@@ -203,8 +204,6 @@ class CustomDataset(Dataset):
     def __len__(self):
         return len(self.image_name)
     
-    def file_name(self):
-        return self.image_name
             
     def __getitem__(self, idx):
         info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
