@@ -10,6 +10,16 @@ import SimpleITK as sitk
 import time
 import os
 
+def get_kernel(patch_size, sigma = 1e+3):
+    kernel_shape = (patch_size[0], patch_size[1], patch_size[2])
+    tmp = tuple([torch.arange(i) for i in kernel_shape])
+    gx, gy, gz = torch.meshgrid(*tmp)
+    gx = torch.abs(gx - kernel_shape[0]//2)
+    gy = torch.abs(gy - kernel_shape[1]//2)
+    gz = torch.abs(gz - kernel_shape[2]//2)
+    kernel = torch.exp(-((gx**2 + gy**2 + gz**2)/(2*(sigma))))
+
+    return kernel
 
 def get_pad_images(images, patch_size):
     #이미지가 patch 보다 작을 경우 padding
@@ -92,21 +102,43 @@ def change_value_from_coords(image, patch_coords, index, patch):
 class CC_Remover(object):
     # remain num => 남길 connected components label의 수
     # frequency order => connected component labeling 된 것 중 택할 빈도 수를 정함. 1은 보통 background 임.
-    def __init__(self, remain_num = 2, frequency_order = [2,3]):
+    def __init__(self, remain_num = 2, frequency_order = [2,3], distance = True):
         self.remain_num = remain_num
         self.frequency_order = frequency_order
+        self.distance = distance
     
 
     def __call__(self, mask):
         conected_component_mask = cc3d.connected_components(mask, connectivity = 26)
         count = np.unique(conected_component_mask, return_counts = True)[1]
         result = torch.zeros(mask.shape)
-        for i in range(self.remain_num):
-            try:
-                result += (np.argsort(count)[::-1][self.frequency_order[i]-1] == conected_component_mask)
-            except IndexError as e:
-                print(e)
-        return result
+
+        if self.distance != False:
+            remained_components_indices = np.argsort(count)[::-1][1:self.remain_num+2]
+            center_coords = []
+            for r_i in remained_components_indices:
+                center_coords.append([np.where(conected_component_mask == r_i)[0].mean(), np.where(conected_component_mask == r_i)[1].mean(), np.where(conected_component_mask == r_i)[2].mean()])
+            center_coords = np.array(center_coords)
+            n_clusters = center_coords.shape[0]
+            distance = np.zeros((n_clusters, n_clusters))
+
+            for i in range(n_clusters):
+                for j in range(n_clusters):
+                    if i == j:
+                        distance[i][j] = 1e+10
+                    else:
+                        distance[i][j] = numpy.linalg.norm(center_coords[i]-center_coords[j])
+
+            x, y = np.unravel_index(np.argmin(distance, axis=None), distance.shape)
+            return torch.tensor((remained_components_indices[x] == conected_component_mask) + (remained_components_indices[y] == conected_component_mask))
+        else:
+            for i in range(self.remain_num):
+                try:
+                    result += (np.argsort(count)[::-1][self.frequency_order[i]-1] == conected_component_mask)
+                except IndexError as e:
+                    print(e)
+                
+            return torch.tensor(result)
 
 class InferenceV2(object):
     """
@@ -118,7 +150,7 @@ class InferenceV2(object):
     padding : patch size에 맞게 padding
     """
     def __init__(self, dataset, model, patch_size, n_classes, mode = 'npy and nifti', step_size = [0.5, 0.5, 0.5], postprocessing = None, device = None,
-                 batch_size = 1, gaussian_filter = False, deep_supervision = False, save_path = None, arg_max = False, padding = False, is_2d = False):
+                 batch_size = 1, gaussian_filter = True, deep_supervision = False, save_path = None, arg_max = False, padding = False, is_2d = False, sigma = 1e+3):
         self.dataset = dataset
         self.model = model.to(device)
         self.bs = batch_size
@@ -135,7 +167,10 @@ class InferenceV2(object):
         self.arg_max = arg_max
         self.padding = padding
         self.is_2d = is_2d
-    
+        if gaussian_filter == True:
+            self.gaussian_kernel = get_kernel(patch_size, sigma = sigma)
+            self.gaussian_kernel = self.gaussian_kernel.to(device)
+        
     def valid_run(self):
         #전체 데이터에 대해 inference 시작.
         print('start valid run')
@@ -162,7 +197,7 @@ class InferenceV2(object):
                 union = temp_seg.sum() + temp_img.sum()
                 dice = 2 * intersection / union
                 dice_score.append(dice.numpy())
-            print(dice_score)
+            print(self.dataset.image_name[data_index], dice_score)
             final_score.append(dice_score)
         
         self.final_score = final_score
@@ -205,7 +240,7 @@ class InferenceV2(object):
             image = self.dataset[data_index][0]
             result = self.get_result(image, data_index, current_name)
             
-        return "Done"
+        return result
     
     def get_result(self, image, data_index = None, current_name = None):
         # 각 이미지 별로 result를 구함.
@@ -230,7 +265,7 @@ class InferenceV2(object):
                 output = self.model(mini_batch_patch.to(self.device).float())
             if self.deep_supervision:
                 output = output[0]
-            output = torch.softmax(output, dim = 1)
+            output = torch.softmax(output, dim = 1) * self.gaussian_kernel.unsqueeze(0).unsqueeze(0)
             output = output.cpu()
             for batch_index in range(mini_batch_index.shape[0]):
                 result = change_value_from_coords(result, coords, mini_batch_index[batch_index], output[batch_index])
@@ -247,14 +282,8 @@ class InferenceV2(object):
             self.save(image[0], self.image_path, data_index, current_name, mode = 'image', is_nifti = True)
 
         if self.postprocessing != None:
-            classes = np.unique(result)
-            new_result = torch.zeros(result.shape)
-    
-            for i in range(len(classes)):
-                class_map = (result.numpy() == classes[i])
-                class_map = self.postprocessing(class_map)
-                new_result += class_map * i
-            result = new_result
+            mask = self.postprocessing((result != 0).detach().cpu().numpy())
+            result = mask * result
         
         self.save(result, self.model_path, data_index, current_name, mode = 'postprocessing', is_nifti = True)
         
