@@ -1,11 +1,56 @@
 import torch.nn as nn
 import torch
 import torch.nn.functional as F
+from torch.utils.checkpoint import checkpoint_sequential
 from ingradient_library.block import *
 
 
+class CheckPointNet(nn.Module):
+    def __init__(self, in_channels = 1, out_channels = 2, n_channels = 32, segment = 8):
+        super().__init__()
+        self.model = nn.Sequential(nn.Conv3d(in_channels, n_channels, kernel_size = 3, stride = 1, padding = 1, dilation = 1),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, n_channels, kernel_size = 3, stride = 1, padding = 24, dilation = 24),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, n_channels, kernel_size = 3, stride = 1, padding = 18, dilation = 18),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, n_channels, kernel_size = 3, stride = 1, padding = 12, dilation = 12),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, n_channels, kernel_size = 3, stride = 1, padding = 6, dilation = 6),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, n_channels, kernel_size = 3, stride = 1, padding = 1, dilation = 1),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, n_channels, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                        nn.Dropout3d(),
+                        nn.InstanceNorm3d(n_channels),
+                        nn.LeakyReLU(),
+                        nn.Conv3d(n_channels, out_channels, kernel_size = 1, stride = 1, padding = 0, dilation = 1)
+                        )
+        self.segment = segment
+
+    def forward(self, X):
+        if self.segment != 0:
+            X = checkpoint_sequential(self.model, self.segment, X)
+        
+        else:
+            X = self.model(X)
+
+        return X
+
 class UNet3d_Plus(nn.Module):
-    def __init__(self, in_channels = 1, out_channels = 2, n_channels = 32, channel_bound = 320):
+    def __init__(self, in_channels = 1, out_channels = 2, n_channels = 32, channel_bound = 320, vectorization = True):
         super().__init__()
         self.expand_channels = encoder_block(in_channels, n_channels, stride = 1)
         self.conv_downsample1 = encoder_block(n_channels, n_channels * 2)
@@ -31,6 +76,7 @@ class UNet3d_Plus(nn.Module):
         self.upsample4 = nn.Upsample(scale_factor=8)
         self.upsample3 = nn.Upsample(scale_factor=4)
         self.upsample2 = nn.Upsample(scale_factor=2)
+        self.vectorization = vectorization
 
     
     def forward(self, x):
@@ -54,6 +100,15 @@ class UNet3d_Plus(nn.Module):
         l3_output = self.upsample3(self.classifier3(l3_output))
         l4_output = self.upsample4(self.classifier4(l4_output))
 
+        if self.vectorization:
+            l1_output = torch.softmax(l1_output , dim = 1)
+            l2_output = torch.softmax(l2_output , dim = 1)
+            l3_output = torch.softmax(l3_output , dim = 1)
+            l4_output = torch.softmax(l4_output , dim = 1)
+            result = torch.stack([l1_output, l2_output, l3_output, l4_output]).permute(1,0,2,3,4,5)
+            
+            return result.contiguous()
+
         
         return [l1_output, l2_output, l3_output, l4_output]
 
@@ -152,8 +207,9 @@ class UNet3d_MultiModal(nn.Module):
 
     
 class UNet3d_MultiModalV2(nn.Module):
-    def __init__(self, in_channels = 1, out_channels = 2, n_channels = 32, channel_bound = 320):
+    def __init__(self, in_channels = 1, out_channels = 2, n_channels = 32, channel_bound = 320, vectorization = True):
         super().__init__()
+        self.vectorization = vectorization
         channel_bound = min(n_channels * 16, channel_bound)
         self.expand_channels_CT = aspp_encoder(in_channels, n_channels, stride = 1)
         self.conv_downsample1_CT = encoder_block(n_channels, n_channels * 2)
@@ -232,6 +288,16 @@ class UNet3d_MultiModalV2(nn.Module):
         l2_output = self.upsample2(self.classifier2(torch.cat((l2_output_CT, l2_output_PET), dim = 1)))
         l3_output = self.upsample3(self.classifier3(torch.cat((l3_output_CT, l3_output_PET), dim = 1)))
         l4_output = self.upsample4(self.classifier4(torch.cat((l4_output_CT, l4_output_PET), dim = 1)))
+
+        if self.vectorization:
+            l1_output = torch.softmax(l1_output , dim = 1)
+            l2_output = torch.softmax(l2_output , dim = 1)
+            l3_output = torch.softmax(l3_output , dim = 1)
+            l4_output = torch.softmax(l4_output , dim = 1)
+            result = torch.stack([l1_output, l2_output, l3_output, l4_output]).permute(1,0,2,3,4,5)
+
+            return result.contiguous()
+
         
         return [l1_output, l2_output, l3_output, l4_output]
 
